from sqlalchemy                 import Column, Integer, String, Text
from sqlalchemy.ext.declarative import declarative_base

from datetime                   import datetime

Base = declarative_base()

class Chapter(Base):
    __tablename__ = 'chapters'
    
    id = Column(Integer, primary_key=True)
    volume = Column(Integer)
    chapter = Column(Integer)
    title = Column(String)
    url = Column(Text)
    mirror_url = Column(Text)  # Alternative URL if telegra.ph is not accessible
    created_at = Column(String, default=lambda: datetime.now().isoformat())

    def __repr__(self):
        return f"<Chapter(volume={self.volume}, chapter={self.chapter}, title={self.title})>"
    
class TocURL(Base):
    __tablename__ = 'ToC_url'

    id = Column(Integer, primary_key=True)
    manga_name = Column(String)
    url = Column(Text)
    mirror_url = Column(Text)
    created_at = Column(String, default=lambda: datetime.now().isoformat())

    def __repr__(self):
        return (
            f"<ToC_url(url={self.url}, "
            f"mirror_url={self.mirror_url}, "
            f"manga_name={self.manga_name})>"
        )


from mangagraph import Mangagraph, MangagraphError
import asyncio

# === tests ===
# python -m tests.__init__
async def test():
    try:
        parser = Mangagraph()
        toc_url, mirror_toc_url = await parser.process_chapter(
            'https://mangalib.me/ru/manga/7965--chainsaw-man',
            chapter_num=90
        )
        print(f"Table of Contents: {toc_url}")
        print(f"Mirror: {mirror_toc_url}")
    except MangagraphError as e:
        print(f"Parser error: {e}")

if __name__ == '__main__':
    try:
        asyncio.run(test())
    except (KeyboardInterrupt, SystemExit):
        print('Sayonara!')


import argparse
import asyncio
import logging

from .parser        import Mangagraph
from .exceptions    import MangagraphError

mgraph = Mangagraph()

async def search_manga(query: str, limit: int = 5):
    results = await mgraph.search_manga(query, limit=limit)
    
    if not results:
        print("Не найдено подходящих манг по запросу.")
        return None

    print(f"\nНайдено {len(results)} по запросу '{query}':")
    for idx, result in enumerate(results, 1):
        print(f"\n{idx}. {result.name} / {result.rus_name}")
        print(f"   Рейтинг: {result.rating.raw_average}/10 ({result.rating.raw_votes} оценок)")
        print(f"   Год: {result.release_year} | Тип: {result.type} | Статус: {result.status}")
        print(f"   Возраст: {result.age_restriction}")

    try:
        choice = int(input(f"\nВыберите мангу (1-{len(results)}, 0 - для отмены): "))
        if choice == 0:
            return None
        if 1 <= choice <= len(results):
            return results[choice-1].slug_url
        else:
            print("Неверный номер.")
            return None
    except ValueError:
        print("Неверный запрос.")
        return None

async def main():
    parser = argparse.ArgumentParser(description="Mangagraph - Manga to Telegraph converter")

    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument('--url', type=str, help='URL манги для обработки')
    group.add_argument('--q', type=str, help='Поиск манги по названию')

    parser.add_argument('--db', type=str, help='Имя БД (по умолчанию - название манги)')
    parser.add_argument('--c', type=int, help='Обработка одной конкретной главы (номер главы, если дан, то вернет ссылку, без создания бд и ToC)')
    parser.add_argument('--limit', type=int, default=5, help='Максимальное количество результатов поиска (по умолчанию 5)')
    
    args = parser.parse_args()
    logger = logging.getLogger(__name__)
    
    try:
        if args.q:
            logger.info(f"Поиск: {args.q}")
            slug = await search_manga(args.q, args.limit)
            
            if not slug:
                logger.info("Поиск отменен или ни одна из манг не выбрана.")
                return
                
            logger.info(f"Выбрана манга: {slug}")
            manga_url = f"https://mangalib.me/ru/manga/{slug}"
        else:
            manga_url = args.url

        if args.c:
            logger.info(f'Обработка главы номер {args.c} | {manga_url}')
            url, mirror_url = await mgraph.process_chapter(manga_url, args.c)
            logger.info(f"Ссылка: {url}")
            logger.info(f"Зеркало: {mirror_url}")
            return 

        logger.info(f"Обработка манги: {manga_url}")
        toc_url, mirror_toc_url = await mgraph.process_manga(manga_url, args.db)
        
        logger.info(f"База данных создана!")
        logger.info(f"Оглавление: {toc_url}")
        logger.info(f"Зеркало оглавления: {mirror_toc_url}")
        
    except MangagraphError as e:
        logger.error(f"Parser error: {e}")
    except KeyboardInterrupt:
        logger.info("Operation canceled by user.")
    except Exception as e:
        logger.error(f"Unexpected error: {e}")

if __name__ == "__main__":
    try:
        asyncio.run(main())
    except RuntimeError:
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        loop.run_until_complete(main())


class MangagraphError(Exception):
    pass

class InvalidURLException(MangagraphError):
    def __init__(self, url, message="Invalid URL provided."):
        self.url = url
        self.message = message
        super().__init__(f"{self.message} URL: {self.url}")

class RequestFailedException(MangagraphError):
    def __init__(self, url, message="Request to the URL failed."):
        self.url = url
        self.message = message
        super().__init__(f"{self.message} URL: {self.url}")


"""
Author: https://github.com/damirtag | https://t.me/damirtag
GH Repo: https://github.com/damirtag/mangagraph

MIT License

Copyright (c) 2025 Tagilbayev Damir

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import asyncio
import aiohttp
import logging

from typing         import List, Dict, Any, Tuple, Optional

from sqlalchemy     import create_engine
from sqlalchemy.orm import sessionmaker, Session

from .models        import Base, Chapter, TocURL
from .schemas       import SearchData
from .exceptions    import (
    MangagraphError,
    RequestFailedException
)
from .utils         import (
    MangaLibUrl, 
    estimate_remaining_time, 
    extract_slug,
    sanitize_db_name
)
from .constants     import (
    TELEGRAPH_CREDS,
    MAX_CONCURRENT,
    API_BASE_URL,
    BASE_IMG_URL,
    HEADERS
)

from telegraph.aio import Telegraph
from telegraph.exceptions import RetryAfterError

class Mangagraph():
    """
    Автор: https://github.com/damirtag

    Репозитории: https://github.com/damirTAG/mangagraph
    """
    def __init__(self):
        self.logger     = self._setup_logger()
        self.telegraph  = Telegraph()
        self.semaphore  = asyncio.Semaphore(MAX_CONCURRENT)

        self.processed_count    = 0
        self.total_chapters     = 0
        self.flood_wait_count   = 0

        self.headers = HEADERS

    def _setup_logger(self):
        logger = logging.getLogger('mangagraph')
        handler = logging.StreamHandler()
        formatter = logging.Formatter('[mangagraph]: %(levelname)s - %(message)s')
        handler.setFormatter(formatter)
        if not logger.handlers:
            logger.addHandler(handler)
        logger.setLevel(logging.INFO)
        return logger

    def _setup_db(self, db_name: str) -> Session:
        try:
            db_name = sanitize_db_name(db_name)
            self.logger.info(f'Имя БД: {db_name}')

            engine = create_engine(f'sqlite:///{db_name}')
            Base.metadata.create_all(engine)

            SessionLocal = sessionmaker(bind=engine)

            return SessionLocal()
        except Exception as e:
            self.logger.error(f'Error trying to setup database: {e}')
            return None

    async def _make_request(
            self, session: aiohttp.ClientSession, url: str, params: Dict = None
        ) -> Dict:
        async with self.semaphore:
            for attempt in range(3):
                try:
                    async with session.get(url, params=params, headers=self.headers) as response:
                        response.raise_for_status()
                        return await response.json()
                except Exception as e:
                    if attempt == 2:
                        raise RequestFailedException(url, str(e))
                    await asyncio.sleep(2 ** attempt)

    async def _get_manga_name(self, session: aiohttp.ClientSession, slug: str) -> tuple[str, str]:
        url = f"{API_BASE_URL}manga/{slug}"
        data: dict = await self._make_request(session, url)
        
        manga_name = data.get('data', {}).get('rus_name', 1)
        if manga_name == 1:
            manga_name = data.get('data', {}).get('name', 'Unknown')
        
        return manga_name

    async def get_chapters_info(
            self, session: aiohttp.ClientSession, slug: str, chapter_num: Optional[float] = None
        ) -> List[Dict[str, Any]]:
        url = f"{API_BASE_URL}manga/{slug}/chapters"
        data = await self._make_request(session, url)

        if not data or 'data' not in data or not isinstance(data['data'], list):
            self.logger.error(f"Похоже, главы в манге были удалены: {slug}")
            return []  # Некоторые манги могут быть с пустыми главами 
            # из-за удаления по требованиям copyright

        chapters = data['data']

        # Фильтруем по номеру главы, если параметр передан
        if chapter_num is not None:
            chapters = [chapter for chapter in chapters if float(chapter.get("number", 0)) == chapter_num]

        return chapters

    async def get_chapter_pages(
        self, 
        session: aiohttp.ClientSession, 
        slug: str, 
        volume: int, 
        chapter: int
    ) -> List[str]:
        url = f"{API_BASE_URL}manga/{slug}/chapter"
        params = {'number': chapter, 'volume': volume}
        data = await self._make_request(session, url, params)
        return [f"{BASE_IMG_URL}{page['url']}" for page in data['data']['pages']]

    async def search_manga(
            self, query: str, limit: int = 5
        ) -> SearchData:
        url = f"{API_BASE_URL}manga"
        params = {
            "fields[]": ["rate_avg", "rate", "releaseDate"],
            "q": query,
            "site_id[]": [1]
        }

        try:
            async with aiohttp.ClientSession() as session:
                async with session.get(url, params=params, headers=self.headers) as response:
                    response.raise_for_status()
                    data = await response.json()
                    
                    results = []
                    for manga_data in data.get('data', [])[:limit]:
                        search_data = SearchData.de_json(manga_data)
                        results.append(search_data)
                    
                    return results
        except aiohttp.ClientError as e:
            print(f"Error during search request: {e}")
            return []
        except Exception as e:
            print(f"Unexpected error during search: {e}")
            return []

    async def _create_telegraph_page(
            self, 
            title: str, 
            image_urls: List[str],
            retry_count: int = 3
        ) -> Tuple[str, str]:
        html_content = "".join(f'<img src="{url}"/>\n' for url in image_urls)
        
        for attempt in range(retry_count):
            try:
                response = await self.telegraph.create_page(
                    title=title,
                    html_content=html_content,
                    author_name='Auto-Generated by MGLParser',
                    author_url='https://github.com/damirTAG/mangagraph'
                )
                
                return (
                    f"https://telegra.ph/{response['path']}", 
                    f"https://graph.org/{response['path']}"
                )
            except RetryAfterError:
                self.flood_wait_count += 1
                wait_time = 7
                            
                self.logger.info(
                    f'Flood wait #{self.flood_wait_count} detected, ' 
                    f'waiting {wait_time} seconds. ' 
                    f'Total floods: {self.flood_wait_count}'
                )
                    
                await asyncio.sleep(wait_time)
                await self.telegraph.create_account(
                    **TELEGRAPH_CREDS
                )
                continue
            except Exception as e:
                raise MangagraphError(f"Не удалось создать telegraph страницу: {str(e)}")

    async def _construct_chapters_list(
        self, 
        title: str, 
        chapters: List[Tuple[int, int, str, str, str]]
    ) -> str:
        """Creates a table of contents page in Telegraph."""
        content = [
            {
                "tag": "p",
                "children": ["Создано Mangagraph, developer - ", {
                    "tag": "a", 
                    "attrs": {"href": "https://t.me/damirtag"}, 
                    "children": ["@damirtag"]}
                ]
            }
        ]
        
        for volume, chapter_num, chapter_title, url, mirror_url in chapters:
            content.append({
                "tag": "p",
                "children": [
                    {
                        "tag": "a",
                        "attrs": {"href": mirror_url},
                        "children": [f"Volume {volume} Chapter {chapter_num}: {chapter_title}"]
                    }
                ]
            })

        try:
            response = await self.telegraph.create_page(
                title=title,
                author_name='Damir',
                author_url='https://github.com/damirtag/mangagraph',
                content=content
            )
            toc_url = response['url']
            mirror_toc_url = toc_url.replace("telegra.ph", "graph.org")

            return toc_url, mirror_toc_url
        except Exception as e:
            raise MangagraphError(f'Telegraph says: {str(e)}')

    async def process_manga(self, manga_url: MangaLibUrl, db_name: str = None):
        """
        Параметры:
            manga_url (str): URL манги, которую нужно обработать.
            db_name (str): Название файла базы данных, по дефолту дается по имени манги

        Возвращает:

            Кортеж из двух строк:

                - URL оглавления в Telegraph.

                - URL зеркала оглавления в graph.org (если используется).

        Исключения:
            MangagraphError: Выбрасывается, если возникает ошибка при обработке манги.
            InvalidURLException: Выбрасывается, если URL манги недействителен.
            RequestFailedException: Выбрасывается, если запрос к API завершается неудачей.
        """
        await self.telegraph.create_account(
            **TELEGRAPH_CREDS
        )
        slug = extract_slug(manga_url)
        
        async with aiohttp.ClientSession() as session:
            try:
                manga_name  = await self._get_manga_name(session, slug)
                chapters    = await self.get_chapters_info(session, slug)
                if not chapters:
                    self.logger.error(f"Похоже, главы в манге были удалены: https://mangalib.me/ru/{slug}")
                    return
                
                db_session: Session  = self._setup_db(
                    manga_name if not db_name else db_name
                )
                if db_session is None:
                    raise RuntimeError("Не удалось создать сессию базы данных.")
                
                self.total_chapters = len(chapters)
                self.processed_count = 0
                processed_chapters = []

                first_chapter = chapters[0]
                pages = await self.get_chapter_pages(
                    session,
                    slug,
                    first_chapter.get('volume'),
                    first_chapter.get('number')
                )
                
                for chapter_info in chapters:
                    volume = chapter_info.get('volume')
                    chapter_num = chapter_info.get('number')
                    title = (
                        f"{manga_name} | {chapter_info.get('name', f'Том {volume}, Глава {chapter_num}')}" 
                    )

                    existing_chapter = db_session.query(Chapter).filter_by(
                        volume=volume, 
                        chapter=chapter_num
                    ).first()

                    if existing_chapter:
                        self.processed_count += 1
                        processed_chapters.append(
                            (volume, chapter_num, title, existing_chapter.url, existing_chapter.mirror_url)
                        )
                        remaining = self.total_chapters - self.processed_count
                        est_time = estimate_remaining_time(remaining)
                        self.logger.info(
                            f"Глава {chapter_num} уже существует, пропускаем... " 
                            f"[{self.processed_count}/{self.total_chapters}] " 
                            f"Примерное время: {est_time}"
                        )
                        continue

                    try:
                        pages = await self.get_chapter_pages(
                            session,
                            slug, 
                            volume, 
                            chapter_num
                        )
                        
                        url, mirror_url = await self._create_telegraph_page(
                            title=title,
                            image_urls=pages
                        )
                        
                        new_chapter = Chapter(
                            volume=volume,
                            chapter=chapter_num,
                            title=title,
                            url=url,
                            mirror_url=mirror_url
                        )
                        db_session.add(new_chapter)
                        db_session.commit()
                        
                        self.processed_count += 1
                        processed_chapters.append((volume, chapter_num, title, url, mirror_url))
                        
                        remaining = self.total_chapters - self.processed_count
                        est_time = estimate_remaining_time(remaining)
                        self.logger.info(
                            f"Обработана глава: {title} " 
                            f"[{self.processed_count}/{self.total_chapters}] " 
                            f"Осталось: {est_time}"
                        )
                        
                        await asyncio.sleep(3)
                        
                    except Exception as e:
                        self.logger.error(
                            f"Ошибка обработки главы {chapter_num}: {str(e)}\n" 
                            f"Всего обработано: {self.processed_count}/{self.total_chapters}"
                        )
                        db_session.rollback()
                        break

                if processed_chapters:
                    toc_url, mirror_toc_url = await self._construct_chapters_list(
                        manga_name,
                        processed_chapters
                    )
                    toc_db = TocURL(
                        manga_name=manga_name,
                        url=toc_url,
                        mirror_url=mirror_toc_url
                    )
                    db_session.add(toc_db)
                    db_session.commit()

                    self.logger.info(f"Создано оглавление: {toc_url}")
                    self.logger.info(f"Всего обработано: {self.processed_count}/{self.total_chapters}")
                    self.logger.info(f"Зеркало: {mirror_toc_url}")

                    return toc_url, mirror_toc_url

            except (KeyboardInterrupt, SystemExit):
                self.logger.info('Sayonara!')

            finally:
                db_session.close()

    async def process_chapter(
            self, manga_url: MangaLibUrl, chapter_num: int
        ) -> Tuple[str, str]:
        """
        Обработка одной главы манги.

        Параметры:
            manga_url (str): URL манги на MangaLib.
            chapter_num (int): Номер главы, которую нужно обработать.

        Возвращает:
            Tuple[str, str] | None: URL страницы Telegraph и ее зеркало, если обработка успешна, иначе None.
        """
        await self.telegraph.create_account(
            **TELEGRAPH_CREDS
        )
        slug = extract_slug(manga_url)

        async with aiohttp.ClientSession() as session:
            try:
                manga_name          = await self._get_manga_name(session, slug)
                chapter_list        = await self.get_chapters_info(session, slug, chapter_num)
                if not chapter_list:
                    raise MangagraphError(f"Похоже, главы в манге были удалены: https://mangalib.me/ru/{slug}")
                chapter = chapter_list[0]

                pages = await self.get_chapter_pages(
                    session,
                    slug,
                    chapter.get('volume'),
                    chapter.get('number')
                )
                if not pages:
                    raise MangagraphError(f"Не удалось получить страницы манги: https://mangalib.me/ru/{slug}")

                volume = chapter.get('volume')
                chapter_num = chapter.get('number')
                title = (
                    f"{manga_name} | {chapter.get('name', f'Том {volume}, Глава {chapter_num}')}" 
                )

                try:
                    url, mirror_url = await self._create_telegraph_page(
                        title=title,
                        image_urls=pages
                    )
                    self.logger.info(f"Обработана глава: {title}")
                    return url, mirror_url
                except Exception as e:
                    self.logger.error(f"Ошибка обработки главы {chapter_num}: {str(e)}\n")
                    return None

            except Exception as e:
                self.logger.error(f'Ошибка при обработке главы: {e}')


from .parser        import Mangagraph
from .models        import Chapter, TocURL
from .schemas       import SearchData
from .exceptions    import (
    MangagraphError,
    InvalidURLException,
    RequestFailedException
)

__all__ = [
    'Mangagraph',
    'Chapter',
    'TocURL',
    'SearchData',
    'MangagraphError',
    'InvalidURLException',
    'RequestFailedException'
]


import dataclasses
from typing import Dict, Any

@dataclasses.dataclass
class Cover:
    thumbnail: str
    default: str
    md: str

@dataclasses.dataclass
class Rating:
    average: float
    votes: int
    raw_average: str
    raw_votes: str

@dataclasses.dataclass
class SearchData:
    id: int
    name: str
    rus_name: str
    eng_name: str
    slug: str
    slug_url: str
    cover: Cover
    age_restriction: str
    type: str
    release_year: str
    rating: Rating
    status: str
    
    @classmethod
    def de_json(cls, data: Dict[str, Any]) -> 'SearchData':
        """Create SearchData instance from API response dictionary."""
        cover_data = data.get('cover', {})
        cover = Cover(
            thumbnail=cover_data.get('thumbnail', ''),
            default=cover_data.get('default', ''),
            md=cover_data.get('md', '')
        )
        
        rating_data = data.get('rating', {})
        rating = Rating(
            average=float(rating_data.get('average', 0)),
            votes=int(rating_data.get('votes', 0)),
            raw_average=rating_data.get('averageFormated', 'N/A'),
            raw_votes=rating_data.get('votesFormated', 'N/A')
        )
        
        return cls(
            id=data.get('id', 0),
            name=data.get('name', 'Unknown'),
            rus_name=data.get('rus_name', ''),
            eng_name=data.get('eng_name', ''),
            slug=data.get('slug', ''),
            slug_url=data.get('slug_url', ''),
            cover=cover,
            age_restriction=data.get('ageRestriction', {}).get('label', 'Unknown'),
            type=data.get('type', {}).get('label', 'Unknown'),
            release_year=data.get('releaseDate', 'Unknown'),
            rating=rating,
            status=data.get('status', {}).get('label', 'Unknown')
        )



import re, hashlib
from urllib.parse   import urlparse
from .exceptions    import InvalidURLException


class MangaLibUrl:
    MANGALIB_URL_PATTERNS = [
        r"^https://mangalib\.me/ru/manga/[\w\-]+(\?.+)?$",
        r"^https://mangalib\.me/ru/\d+--[\w\-]+/read/v\d+/c\d+(\?.+)?$"
    ]
    
    def __init__(self, url: str):
        self.url = url
        self._validate_url()

    def _validate_url(self):
        parsed_url = urlparse(self.url)
        if not (parsed_url.scheme == "https" and parsed_url.netloc == "mangalib.me"):
            raise ValueError("Неверный адресс. Только 'https://mangalib.me' доступен.")
        
        if not any(re.compile(pattern).match(self.url) for pattern in self.MANGALIB_URL_PATTERNS):
            raise ValueError(
                "Неверный URL. Ссылка должна быть одного из типов:\n"
                "- 'https://mangalib.me/ru/manga/{slug_url}'\n"
                "- 'https://mangalib.me/ru/{slug_url}/read/v{volume}/c{chapter}'"
            )

    def __str__(self):
        return self.url

def estimate_remaining_time(remaining_chapters: int, chapters_per_minute: int = 12) -> str:
    estimated_minutes = (remaining_chapters / chapters_per_minute) * 1.1
    
    if estimated_minutes < 1:
        return "меньше минуты"
        
    hours = int(estimated_minutes // 60)
    minutes = int(estimated_minutes % 60)
    
    if hours > 0:
        return f"{hours} ч {minutes} мин"
    return f"{minutes} мин"

def extract_slug(url: str) -> str:
    pattern = r'/manga/(\d+--[^?]+)'
    match = re.search(pattern, url)
    if match:
        return match.group(1)
    raise InvalidURLException(url, "Invalid MangaLib URL format.")

def sanitize_db_name(db_name: str) -> str:
    clean_name = re.sub(r'[^\w\-.]', '_', db_name)
    
    if len(clean_name) < 3 or clean_name == '_' * len(clean_name):
        hash_suffix = hashlib.md5(db_name.encode()).hexdigest()[:8]
        safe_name = f"{hash_suffix}.db"
    else:
        safe_name = clean_name + '.db'
    return safe_name


from setuptools import setup, find_packages

import pathlib


LIB_NAME = 'mangagraph'

WORK_DIR = pathlib.Path(__file__).parent

VERSION = '0.0.3.post5'

setup(
    name=LIB_NAME,
    version=VERSION,
    description='Async manga parser-converter from mangalib to telegraph pages',
    long_description = (WORK_DIR / "readme.md").read_text("utf-8"),
    long_description_content_type='text/markdown',
    url='https://github.com/damirTAG/mangagraph',
    author='damirTAG',
    author_email='damirtagilbayev17@gmail.com',
    packages=find_packages(),
    install_requires=[
        'aiohttp',
        'sqlalchemy',
        'telegraph',
        'asyncio'
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
    ],
    keywords=[
        'mangalib',
        'mangalib-parser',
        'manga',
        'telegraph'
    ],
    python_requires='>=3.7',
    include_package_data=False
)


MAX_CONCURRENT      = 3
CHAPTERS_PER_MINUTE = 20
BASE_IMG_URL        = "https://img2.imglib.info"
API_BASE_URL        = "https://api2.mangalib.me/api/"
TELEGRAPH_CREDS     = dict(
    short_name  ='Damir',
    author_name ='Создано mangagraph',
    author_url  ='https://github.com/damirTAG/mangagraph'
)

HEADERS = {
    'Authorization' :   'Bearer eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJhdWQiOiIxIiwianR'
                        'pIjoiNWE5YmI5MDVmNjlmNzJkYTkxN2Q5YjY4OTU3ZWI2ODI0MDFkYjUyYjFhNzhjMDk0ZDg3MWExN2Qy'
                        'MmFjYjk2ZTg5YWM4ZjJmN2Q2YjBkNTMiLCJpYXQiOjE3NDE4MTI2NjEuODMzNTU0LCJuYmYiOjE3NDE4M'
                        'TI2NjEuODMzNTU2LCJleHAiOjE3NDQ0OTEwNjEuODI5NzY4LCJzdWIiOiI3MDExNTkwIiwic2NvcGVzIj'
                        'pbXX0.teJSwDcP0jp2yuuJLM5kFH3f89qVHOEOvKws70gvc6oo9Gd-dBLt8XSMipL-5ypHCFFTb5qfj9i'
                        '2xvquUpA618JMDyP31wwfZvaa3h8vVvdeEeWjf98xMpjq8ngMGioSJiWc4WipFmF7sXDpWPvKN-6ECEG8V'
                        'oYA0bd7FhxdGYeJS5GrK2tcdMxHoCtIW8HrMdo0LTYtqtz3c5bq_FnrRljqrdxU2GyxTI7QxRk81X5xZK'
                        'XBkvSEmm1l3DEJPkIP22KjegOC5jQzh_sv3GN4yHWMr7m2py5_A2Gwgm7hbH-q9a_Aw8CB-mripbxSqug'
                        '0Z6yT4kcWxd0AePLztGdTQx8Gd9oPqP9twDZIJnwguyQGOBQ-DHRjYrl0XSAn89OQIbcvQtveSk7vY0_8'
                        'l3COUGeRkNBBg2BT-C0jYOb7SVb_tn59VAmm5a2Z9ZdCyXhzfqVoe9yw1I0DlefUpxeFSyfo9G7ktxTep'
                        'sWY0NMW9S9I_PFg7WLlXX_0jwtbAcuQ6AA1QRiLW-QvAR5cYnbq7jkreF-o_Ja8XxIjRNKuemDtj9feTt'
                        'shGWxnfMYPJcfRFaVxpGlon6nNL9qVFJmB8UrqGeFUqwjlZ0PFAsvVxN459il1YOysYObG6OhW1IcKWtK'
                        'Px5lkXjbH5UqcDqRYAIxPT1uVWU2npmgGE6gq_9U',
    'User-Agent'    :   'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/134.0.0.0 Safari/537.36'
}


from .cli import main
import asyncio


if __name__ == "__main__":
    try:
        asyncio.run(main())
    except RuntimeError: 
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        loop.run_until_complete(main())